;Filename: AdcGo.s
;Part: ADuC702x.
;Created: 15/04/2004
;Updated: 28/06/2005
;Author: Eckart Hartmann.
;This file is in beta sampling.

#include        <ADuC702x.IAR.equ>


; AdcGo==========Start ADC conversion.
; C Function prototype: int AdcGo(int iChanP, int iChanN, int iStart);
;iChanP:{ADC_0,ADC_1,ADC_2,ADC_3,ADC_4,ADC_5,ADC_6,ADC_7,ADC_8,ADC_9,ADC_10,ADC_11,ADC_12,ADC_13,ADC_14,ADC_15,ADC_TEMP,ADC_GND,ADC_REFP,ADC_VDD2}
;iChanN:{ADC_0,ADC_1,ADC_2,ADC_3,ADC_4,ADC_5,ADC_6,ADC_7,ADC_8,ADC_9,ADC_10,ADC_11,ADC_12,ADC_13,ADC_14,ADC_15,ADC_REFN,ADC_GND,ADC_NEG}
;iStart:{ADC_OFF,ADC_T2S,ADC_T3S,ADC_SWS,ADC_CCS,ADC_PLAS,ADC_IRQS}
; Description of Function: Start ADC conversion.
; User interface:
;   Set iChanP to set which signal will be connected to the positive input of the ADC:
;       0-15 or ADC_0-ADC15 for external channels 0-15 (if available).
;       16 or ADC_TEMP for internal temperature (iChanN must be 17 (ADC_GND)).
;       17 or ADC_GND for ground.
;       18 or ADC_REFP for positive reference voltage.
;       19 or ADC_VDD2 for AVDD/2.
;       other value same as ADC_GND.
;   Set iChanN to set which signal will be connected to the negative input of the ADC:
;       0-15 or ADC_0-ADC15 for external channels 0-15 (if available).
;       16 or ADC_REFN for negative reference voltage.
;       17 or ADC_GND for single ended conversion.
;       18 or ADC_NEG for pseudo differential conversion.
;       other value same as ADC_NEG.
;   Set iStart to one of the following:
;       0 or ADC_IRQS for IRQ0 pin to start a conversion.
;       1 or ADC_T2S for T2 timer to start a conversion.
;       2 or ADC_T3S for T3 timer to start a conversion.
;       3 or ADC_SWS to start a single conversion.
;       4 or ADC_CCS to start continuous conversions.
;       5 or ADC_PLAS for the PLA to start a conversion.
;       other value to power down ADC (ADC_OFF).
;   Call AdcGo to start converting with these settings.
;       Returns 0 if ADC had been powered down.
; Robustness:  The first conversion result after the ADC had been powered
;               down (ADC_OFF) is not accurate.
; Side effects: Corrupts r3 and r1.
;               Terminates any conversion that might be running.

                RSEG CODE
                MODULE AdcGo
                PUBLIC AdcGo
                CODE32

AdcGo:
        mov     r3,#0xff0000            ;r3 = Z_BASE;
        orr     r3,r3,#0xff000000
        cmp     r0,#20                  ;if(iChanP>=20)
        movhs   r0,#17                  ; iChanP = 17;
        str     r0,[r3,#Z_ADCCP]        ;ADCCP = iChanP;

        cmp     r1,#17                  ;if(iChanN<17)
        strlo   r1,[r3,#Z_ADCCN]        ; ADCCN = iChanN;

        ldr     r1,[r3,#Z_ADCCON]       ;r1 = ADCCON
        mov     r0,r1,LSL#26            ;r0 = ADCCON.5; //For return.
        mov     r0,r0,LSR#31
        and     r1,r1,#0x40             ;Remove bits to change.
        orrlo   r1,r1,#8                ;if(iChanN<17) make differential
        orrhi   r1,r1,#0x10             ;if(iChanN>17) make pseudo differential.

        cmp     r2,#6                   ;if(iStart>=6)
        andhs   r1,r1,#0xdf             ; clear power_bit.
        orrlo   r1,r1,#0xa0             ;else set power_bit and enable
        orrlo   r1,r1,r2                ; and add iStart.
        str     r1,[r3,#Z_ADCCON]       ;ADCCON = r1;
        bx      lr                      ;return power_bit.

                 ENDMOD AdcGo

                 END

